<?php

/**
 * @file
 * Contains \Drupal\sharemessage\Plugin\sharemessage\Sharrre.
 */

namespace Drupal\sharemessage\Plugin\sharemessage;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Template\Attribute;
use Drupal\sharemessage\SharePluginBase;
use Drupal\sharemessage\SharePluginInterface;

/**
 * Sharrre plugin.
 *
 * @SharePlugin(
 *   id = "sharrre",
 *   label = @Translation("Sharrre"),
 *   description = @Translation("Sharrre is a jQuery plugin that allows you to
 *   create nice widgets sharing for Facebook, Twitter, Google Plus
 *   (with PHP script) and more. (http://sharrre.com)"),
 * )
 */
class Sharrre extends SharePluginBase implements  SharePluginInterface {

  /**
   * Check if the plugin can work.
   *
   * The library needs to be either external or we need the local library module with the library present.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array
   */
  public static function checkConfiguration(array $form, $show_message = FALSE) {
    if (\Drupal::config('sharemessage.sharrre')->get('library_url')) {
      // We have an extrnal library URL configured. Fine.
      return $form;
    }

    // Check for both, local library and external URL.
    if (!\Drupal::moduleHandler()->moduleExists('libraries') && !\Drupal::config('sharemessage.sharrre')->get('library_url') && $show_message) {
      $form['message'] = array(
        '#type' => 'container',
        '#markup' => t('Either set the library locally and enable the libraries module or enter the external URL.'),
        '#attributes' => array(
          'class' => array('messages messages--warning'),
        ),
      );
      return $form;
    }

    if (\Drupal::moduleHandler()->moduleExists('libraries')) {
      // Check if local library is set correctly.
      $directory = libraries_get_path('sharrre');
      $file = 'jquery.sharrre.min.js';
      if (!file_exists($directory . '/' . $file) && $show_message) {
        $form['message'] = array(
          '#type' => 'container',
          '#markup' => t('The library file is not present in the directory.'),
          '#attributes' => array(
            'class' => array('messages messages--warning'),
          ),
        );
        return $form;
      }
    }
    else {
      // Without the libraries module, the external url is required.
      $form['message'] = array(
        '#type' => 'container',
        '#markup' => t('Without a local library, you need to set the external URL.'),
        '#attributes' => array(
          'class' => array('messages messages--warning'),
        ),
      );
      return $form;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function build($context, $plugin_attributes) {

    $attributes = new Attribute(array('id' => array(
      'sharemessage',
    )));

    if ($plugin_attributes) {
      $attributes['sharrre:url'] = $this->shareMessage->getUrl($context);
      $attributes['sharrre:title'] = $this->shareMessage->getTokenizedField($this->shareMessage->title, $context);
      $attributes['sharrre:description'] = $this->shareMessage->getTokenizedField($this->shareMessage->message_long, $context);
    }

    // Add Sharrre buttons.
    $build = array(
      '#theme' => 'sharemessage_sharrre',
      '#attributes' => $attributes,
      '#attached' => array(
        'library' => ['sharemessage/sharrre'],
        'drupalSettings' => array(
          'sharrre_config' => array(
            'services' => $this->shareMessage->getSetting('services') ?: \Drupal::config('sharemessage.sharrre')->get('services'),
            'library_url' => $this->shareMessage->getSetting('library_url') ?: \Drupal::config('sharemessage.sharrre')->get('library_url'),
            'shorter_total' => $this->shareMessage->getSetting('shorter_total') ?: \Drupal::config('sharemessage.sharrre')->get('shorter_total'),
            'enable_hover' => $this->shareMessage->getSetting('enable_hover') ?: \Drupal::config('sharemessage.sharrre')->get('enable_hover'),
            'enable_counter' => $this->shareMessage->getSetting('enable_counter') ?: \Drupal::config('sharemessage.sharrre')->get('enable_counter'),
            'enable_tracking' => $this->shareMessage->getSetting('enable_tracking') ?: \Drupal::config('sharemessage.sharrre')->get('enable_tracking'),
            'url' => $this->shareMessage->share_url,
          ),
        ),
      ),
    );
    return $build;
  }

  /**
   * {@inheritdoc}
   */
  function getSetting($key) {
    $override = $this->shareMessage->getSetting('override_default_settings');
    if (isset($override)) {
      return $this->shareMessage->getSetting($key);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {

    // Check if the library configuration is valid.
    static::checkConfiguration($form, TRUE);

    // Settings fieldset.
    $form['override_default_settings'] = array(
      '#type' => 'checkbox',
      '#title' => t('Override default settings'),
      '#default_value' => $this->getSetting('override_default_settings'),
    );

    $form['services'] = array(
      '#title' => t('Visible services'),
      '#type' => 'select',
      '#multiple' => TRUE,
      '#options' => array (
        'googlePlus' => $this->t('Google+'),
        'facebook' => $this->t('Facebook'),
        'twitter' => $this->t('Twitter'),
        'digg' => $this->t('Digg'),
        'delicious' => $this->t('Delicious'),
        'stumbleupon' => $this->t('StumpleUpon'),
        'linkedin' => $this->t('Linkedin'),
        'pinterest' => $this->t('Pinterest'),
      ),
      '#default_value' => $this->getSetting('services'),
      '#size' => 10,
      '#states' => array(
        'invisible' => array(
          ':input[name="settings[override_default_settings]"]' => array('checked' => FALSE),
        ),
      ),
    );

    $form['shorter_total'] = array(
      '#type' => 'checkbox',
      '#title' => t('Format number like 1.2k or 5M'),
      '#default_value' => $this->getSetting('shorter_total'),
      '#states' => array(
        'invisible' => array(
          ':input[name="settings[override_default_settings]"]' => array('checked' => FALSE),
        ),
      ),
    );

    $form['enable_hover'] = array(
      '#type' => 'checkbox',
      '#title' => t('Allow the sharing buttons'),
      '#default_value' => $this->getSetting('enable_hover'),
      '#states' => array(
        'invisible' => array(
          ':input[name="settings[override_default_settings]"]' => array('checked' => FALSE),
        ),
      ),
    );

    $form['enable_counter'] = array(
      '#type' => 'checkbox',
      '#title' => t('Enable the total counter'),
      '#default_value' => $this->getSetting('enable_counter'),
      '#states' => array(
        'invisible' => array(
          ':input[name="settings[override_default_settings]"]' => array('checked' => FALSE),
        ),
      ),
    );

    $form['enable_tracking'] = array(
      '#type' => 'checkbox',
      '#title' => t('Allow tracking social interaction with Google Analytics'),
      '#default_value' => $this->getSetting('enable_tracking'),
      '#states' => array(
        'invisible' => array(
          ':input[name="settings[override_default_settings]"]' => array('checked' => FALSE),
        ),
      ),
    );

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    $sharrre_config = \Drupal::config('sharemessage.sharrre');
    if (!\Drupal::moduleHandler()->moduleExists('libraries') && empty($sharrre_config->get('library_url')) && empty($form_state->getValue('settings')['library_url'])) {
      $form_state->setErrorByName('plugin', t('Either set the library locally and enable the libraries module or enter the external URL.'));
    }
    if (\Drupal::moduleHandler()->moduleExists('libraries')) {
      $directory = libraries_get_path('sharrre');
      $file = 'jquery.sharrre.min.js';
      if (!file_exists($directory . '/' . $file)) {
        $form_state->setErrorByName('plugin', t('The library file %file is not present in the directory %directory.', array(
          '%file' => $file,
          '%directory' => $directory
        )));
      }
    }
    else {
      if (empty($sharrre_config->get('library_url')) && empty($form_state->getValue('settings')['library_url'])) {
        $form_state->setErrorByName('plugin', t('Without a local library, you need to set the external URL.'));
      }
    }
  }

}
