<?php
/**
 * @file
 * Definition of Drupal\sharemessage\Tests\ShareMessageSettingsTest.
 */

namespace Drupal\sharemessage\Tests;

/**
 * Check if default and sharemessage specific settings work correctly.
 *
 * @group sharemessage
 */
class ShareMessageSettingsTest extends ShareMessageTestBase {

  /**
   * Test case that check if default and sharemessage specific settings work correctly.
   */
  public function testShareMessageSettings() {

    // Step 1: Setup default settings.
    $this->drupalGet('admin/config/services/sharemessage/addthis-settings');
    $default_settings = array(
      'default_services[]' => array(
        'facebook',
        'facebook_like',
      ),
      'default_additional_services' => FALSE,
      'default_icon_style' => 'addthis_16x16_style',
    );
    $this->drupalPostForm(NULL, $default_settings, t('Save configuration'));

    // Step 2: Create share message with customized settings.
    $this->drupalGet('admin/config/services/sharemessage/add');
    $sharemessage = array(
      'label' => 'ShareMessage Test Label',
      'id' => 'sharemessage_test_label',
      'settings[override_default_settings]' => 1,
      'settings[services][]' => array(
        'facebook',
      ),
      'settings[additional_services]' => 1,
      'settings[icon_style]' => 'addthis_32x32_style',
    );
    $this->drupalPostForm(NULL, $sharemessage, t('Save'));
    $this->assertText(t('ShareMessage @label has been added.', array('@label' => $sharemessage['label'])));

    // Step 3: Verify that settings are overridden
    // (services, additional_services and icon_style).
    $this->drupalGet('sharemessage-test/sharemessage_test_label');
    $raw_html_services = '<a class="addthis_button_facebook_like"></a>';
    $raw_html_additional_services = '<a class="addthis_button_compact"></a>';
    $raw_html_icon_style = '<div class="addthis_toolbox addthis_default_style ' . $sharemessage['settings[icon_style]'] . '"';

    // Check services (facebook_like button should not be displayed).
    $this->assertNoRaw($raw_html_services, 'Facebook like button that is globally enabled is not displayed on the page, so that the global settings are overridden.');

    // Additional services should be displayed.
    $this->assertRaw($raw_html_additional_services, 'Additional services button is displayed, so that the global settings are overridden.');

    // Check icon style.
    $this->assertRaw($raw_html_icon_style, 'Icon style is changed to "' . $sharemessage['settings[icon_style]'] . '" so that the global settings are overridden.');

    // Step 4: Uncheck "Override default settings" checkbox.
    $this->drupalGet('admin/config/services/sharemessage/manage/' . $sharemessage['id']);
    $edit = array(
      'settings[override_default_settings]' => FALSE,
    );
    $this->drupalPostForm(NULL, $edit, t('Save'));
    $this->assertText(t('ShareMessage @label has been updated.', array('@label' => $sharemessage['label'])));

    // Step 5: Check that addThis widget is displayed with default settings.
    $this->drupalGet('sharemessage-test/sharemessage_test_label');

    // Check services (facebook_like button should be displayed).
    $this->assertRaw($raw_html_services, 'Facebook like button is displayed as it is globally configured.');

    // Additional services button should not be displayed.
    $this->assertNoRaw($raw_html_additional_services, 'Additional services buttion is not displayed as it is globally configured.');

    // Check icon style (should be addthis_16x16_style).
    $raw_html_default_icon_style = '<div class="addthis_toolbox addthis_default_style ' . $default_settings['default_icon_style'] . '"';
    $this->assertRaw($raw_html_default_icon_style, 'Default icon style is used.');

    // Step 1: Setup sharrre plugin settings.
    $this->drupalGet('admin/config/services/sharemessage/sharrre-settings');

    // Check if configuration is incorrect (library is missing).
    $this->assertText('Either set the library locally and enable the libraries module or enter the external URL.');

    $default_settings = array(
      'default_services[]' => array(
        'googlePlus',
        'facebook',
        'twitter',
        'linkedin',
        'pinterest',
      ),
      'library_url' => '//cdn.jsdelivr.net/sharrre/1.3.4/jquery.sharrre-1.3.4.min.js',
      'shorter_total' => TRUE,
      'enable_hover' => TRUE,
      'enable_counter' => FALSE,
    );
    $this->drupalPostForm(NULL, $default_settings, t('Save configuration'));

    // Step 2: Switch to Sharrre plugin.
    $this->drupalGet('admin/config/services/sharemessage');
    $this->clickLink('Edit');
    $sharemessage = array(
      'label' => 'ShareMessage Sharrre Test Label',
      'plugin' => 'sharrre',
      'title' => 'Sharrre',
      'message_long' => 'Test long message',
      'message_short' => 'Test short message',
    );
    $this->drupalPostForm(NULL, $sharemessage, t('Save'));
    $this->assertText(t('ShareMessage @label has been updated.', array('@label' => $sharemessage['label'])));

    $this->drupalGet('sharemessage-test/sharemessage_test_label');
    $this->assertRaw('"library_url":"\/\/cdn.jsdelivr.net\/sharrre\/1.3.4\/jquery.sharrre-1.3.4.min.js"');
    $this->assertRaw('"googlePlus":"googlePlus","facebook":"facebook","twitter":"twitter","linkedin":"linkedin","pinterest":"pinterest"');
  }

  /**
   * Test case for the delete and cancel button functionality for ShareMessage.
   */
  function testShareMessageDeleteCancel() {
    // Create a share message.
    $this->drupalGet('admin/config/services/sharemessage/add');
    $sharemessage = [
      'label' => 'ShareMessage Test Label',
      'id' => 'sharemessage_test_label',
      'settings[override_default_settings]' => 1,
      'settings[services][]' => [
        'facebook',
      ],
      'settings[additional_services]' => 1,
      'settings[icon_style]' => 'addthis_32x32_style',
    ];
    $this->drupalPostForm(NULL, $sharemessage, t('Save'));

    // Check newly created ShareMessage on list page.
    $this->drupalGet('admin/config/services/sharemessage');
    $this->assertText($sharemessage['label'], 'Newly created sharemessage found.');
    // Check for Edit link.
    $this->assertLink('Edit');
    // Check for the Delete link.
    $this->assertLink('Delete');

    // Click delete link on admin ui.
    $this->clickLink('Delete');
    $this->assertText(t('Are you sure you want to delete the share message @label?', ['@label' => $sharemessage['label']]));

    // Check if cancel button is present or not.
    $this->assertLink('Cancel');

    // Delete the share message.
    $this->drupalPostForm(NULL, [], t('Delete'));
    $this->assertText(t('The share message @label has been deleted.', ['@label' => $sharemessage['label']]));

    // Check if removed from listing page as well.
    $this->drupalGet('admin/config/services/sharemessage');
    $this->assertNoText($sharemessage['label'], 'Not found the deleted message.');
  }

  /**
   * Test case for OGHeaders plugin.
   */
  public function testShareMessageOGTags() {
    // Create share message with OG headers as plugin.
    $this->drupalGet('admin/config/services/sharemessage/add');
    $sharemessage = array(
      'label' => 'ShareMessage Test OG Label',
      'id' => 'sharemessage_test_og_label',
      'plugin' => 'ogheaders',
      'title' => 'OG headers name',
      'message_long' => 'OG headers long description',
      'message_short' => 'OG headers short description',
    );
    $this->drupalPostForm(NULL, $sharemessage, t('Save'));
    $this->assertText(t('ShareMessage @label has been added.', array('@label' => $sharemessage['label'])));
    $this->drupalGet('sharemessage-test/sharemessage_test_og_label');

    // Check for OG headers in meta tags.
    $this->assertRaw('<meta property="og:title" content="OG headers name" />');
    $this->assertRaw('<meta property="og:url" content="' . $this->url . '" />');
    $this->assertRaw('<meta property="og:description" content="OG headers long description" />');
  }

  /**
   * Test case for special characters encoding.
   */
  public function testShareMessageSpecialCharsEncoding() {
    // Create share message with OG headers as plugin.
    $this->drupalGet('admin/config/services/sharemessage/add');
    $sharemessage = array(
      'label' => 'Special characters encoding test',
      'id' => 'sharemessage_test_special_chars',
      'plugin' => 'addthis',
      'title' => 'Inondations sur la Côte d\'Azur: «C’est apocalyptique, c’est Tchernobyl»',
      'message_long' => 'Long description',
      'message_short' => 'Short description',
    );
    $this->drupalPostForm(NULL, $sharemessage, t('Save'));
    $this->assertText(t('ShareMessage @label has been added.', array('@label' => $sharemessage['label'])));
    $this->drupalGet('sharemessage-test/sharemessage_test_special_chars');
    // Check for correct encoding in meta tags.
    $this->assertRaw('<meta property="og:title" content="Inondations sur la Côte d&#039;Azur: «C’est apocalyptique, c’est Tchernobyl»" />');
    $this->assertNoRaw('<meta property="og:title" content="Inondations sur la Côte d\'Azur: «C’est apocalyptique, c’est Tchernobyl»" />');
    $this->assertRaw('<meta property="og:url" content="' . $this->url . '" />');
    $this->assertRaw('<meta property="og:description" content="Long description" />');
    $this->assertRaw('<meta property="og:type" content="website" />');

    $this->drupalGet('admin/config/services/sharemessage/add');
    $sharemessage2 = array(
      'label' => 'Special characters encoding test 2',
      'id' => 'sharemessage_test_special_chars_2',
      'plugin' => 'addthis',
      'title' => 'This is a second test with quotes "',
      'message_long' => 'Long description 2',
      'message_short' => 'Short description 2',
    );
    $this->drupalPostForm(NULL, $sharemessage2, t('Save'));
    $this->drupalGet('sharemessage-test/sharemessage_test_special_chars_2');
    $this->assertRaw('<meta property="og:title" content="This is a second test with quotes &quot;" />');
    $this->assertNoRaw('<meta property="og:title" content="This is a second test with quotes "" />');
  }
}
